<html>
<head>
<meta charset="utf-8"/>
<script>
window.mainUrl = "{{ MAIN_URL }}";
</script>
<script src="./load.js"></script>
</head>
<body>
<div id="loading" style="width: 100%; text-align: center">
    <svg width="38" height="38" viewBox="0 0 38 38" xmlns="http://www.w3.org/2000/svg" stroke="#fff">
        <g fill="none" fill-rule="evenodd">
            <g transform="translate(1 1)" stroke-width="2">
                <path d="M36 18c0-9.94-8.06-18-18-18" stroke="gray">
                    <animateTransform
                        attributeName="transform"
                        type="rotate"
                        from="0 18 18"
                        to="360 18 18"
                        dur="1s"
                        repeatCount="indefinite"/>
                </path>
            </g>
        </g>
    </svg>
</div>
<div id="error"></div>
</body>
</html>
